"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const asynciterator_1 = require("asynciterator");
/**
 * An iterator that takes elements from a given set of iterators in a round-robin manner.
 *
 * Based on LDF client's UnionIterator:
 * https://github.com/LinkedDataFragments/Client.js/blob/master/lib/sparql/UnionIterator.js
 */
class RoundRobinUnionIterator extends asynciterator_1.BufferedIterator {
    constructor(sources, options) {
        super(options);
        this.currentSource = 0;
        this.sources = sources;
        for (const source of this.sources) {
            source.on('readable', () => this._fillBuffer());
            source.on('end', () => this._fillBuffer());
            source.on('error', (error) => this.emit('error', error));
        }
    }
    _read(count, done) {
        let item = null;
        let attempts = this.sources.length;
        // Iterate over all sources once
        while (this.sources.length && item === null && attempts--) {
            const source = this.sources[this.currentSource];
            // Read from the current source
            item = source.read();
            // Remove the source if it has ended, otherwise, increment our stored position
            if (source.ended) {
                this.sources.splice(this.currentSource, 1);
            }
            else {
                this.currentSource++;
            }
            this.currentSource = this.currentSource % this.sources.length;
        }
        // Push to the buffer if we have an item
        if (item !== null) {
            this._push(item);
        }
        // Otherwise close
        if (!this.sources.length) {
            this.close();
        }
        done();
    }
}
exports.RoundRobinUnionIterator = RoundRobinUnionIterator;
//# sourceMappingURL=RoundRobinUnionIterator.js.map